
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.Conflict;
import de.codedo.conflicteditor.ConflictsView;
import de.codedo.conflicteditor.CouchDb;
import de.codedo.conflicteditor.Difference;
import de.codedo.conflicteditor.DocumentMatcher;
import de.codedo.conflicteditor.UrlConnectionHttpAccess;

import java.awt.event.ActionEvent;
import java.io.Reader;
import java.net.URL;
import java.util.List;

import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;

public class ConflictEditor
{
    private ConflictEditorFrame _frame;
    private CouchDb _database;

    public static void main(String[] args)
    {
        try
        {
            new ConflictEditor().run();
        }
        catch (Exception ex)
        {
            System.out.println("Uncaught toplevel Exception");
            ex.printStackTrace(System.out);
        }
    }

    private void run()
    {
        _frame = new ConflictEditorFrame();
        connectListeners(_frame);
        _frame.pack();
        _frame.setVisible(true);
    }

    private void connectListeners(ConflictEditorFrame frame)
    {
        frame.getConnectButton().addActionListener(
            new ExceptionHandlingActionListener(new Executable<ActionEvent>()
            {
                @Override
                public void execute(ActionEvent event) throws Exception
                {
                    connectAndFindConflicts();
                }
            }));

        frame.getConflictsList().addListSelectionListener(
            new ExceptionHandlingListSelectionListener(new Executable<ListSelectionEvent>()
            {
                @Override
                public void execute(ListSelectionEvent event) throws Exception
                {
                    if (event.getValueIsAdjusting() == false)
                    {
                        selectConflictDocument(event.getFirstIndex());
                    }
                }
            }));

        frame.getDifferencesTable().getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            @Override
            public void valueChanged(ListSelectionEvent e)
            {
                selectDifference();
            }
        });
    }

    protected void connectAndFindConflicts() throws Exception
    {
        String dbUrl = _frame.getDatabaseUrl();
        _database = new CouchDb(dbUrl);
        ConflictsView conflictsView = new ConflictsView(_database);

        ListModel model = new ConflictsListModel(conflictsView.getConflicts());
        _frame.getConflictsList().setModel(model);
    }

    protected void selectConflictDocument(int index) throws Exception
    {
        JsonNode conflictNode = (JsonNode)_frame.getConflictsList().getModel().getElementAt(index);
        Conflict conflict = new Conflict(conflictNode);
        JsonNode currentDocument = conflict.getCurrentDocument();

        URL documentUrl = _database.urlForDocumentAndRevision(conflict.getId(),
            conflict.getConflictRevision());
        Reader reader = new UrlConnectionHttpAccess(documentUrl).get();

        JsonNode conflictDocument = new ObjectMapper().readTree(reader);

        List<Difference> diffs = new DocumentMatcher(currentDocument, conflictDocument).compare();

        TableModel model = new DifferenceTableModel(diffs);
        _frame.getDifferencesTable().setModel(model);
    }

    protected void selectDifference()
    {
        System.out.println("select a difference");
    }
}
